/*
 * Copyright 2005, 2007 Frank Mayhar.  All Rights Reserved.
 * Borrowed heavily from "ffsrecov," copyright 1999, 2001 John-Mark Gurney.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: block.c,v 1.3 2007/02/06 05:21:50 frank Exp $
 *
 */

#include <fcntl.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <err.h>
#include <timeconv.h>

#include <sys/types.h>
#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/mman.h>
#include <sys/mount.h>
#include <ufs/ufs/ufsmount.h>
#include <ufs/ufs/dinode.h>
#include <ufs/ffs/fs.h>
#include <libufs.h>

#define NO_BL_EXTERNS
#include "ffs2recov.h"

enum parts { P_DMIN, P_IMIN, P_SBLOCK, P_TOD };

off_t blocktooffset( struct uufsd *, ufs2_daddr_t);
void findblk(struct uufsd *, ufs2_daddr_t, ino_t *);

off_t
blocktooffset(
	struct uufsd *disk,
	ufs2_daddr_t blk)
{
	off_t a, b;

	a = fsbtodb(&disk->d_fs, blk);
	b = a * DEV_BSIZE;
	return(b);
}

void
findblk(
	struct uufsd *disk,
	ufs2_daddr_t blk,
	ino_t *num)
{
	int i;
	int diff;
	int tdiff;
	int ilen;
	enum parts type;
	struct ufs2_dinode *inode;
	ufs2_daddr_t iblk;

	ilen = roundup2(sizeof(struct ufs2_dinode), disk->d_bsize);
	inode = malloc(ilen);
	/*
	 * convert to file system block 
	 */
	blk = dbtofsb(&disk->d_fs, blk);

	for (i = disk->d_fs.fs_ncg - 1; i >= 0 && cgbase(&disk->d_fs, i) > blk;
									    i--)
		;

	diff = blk - cgdmin(&disk->d_fs, i);
	type = P_DMIN;
	if (diff > (tdiff = blk - cgimin(&disk->d_fs, i))) {
		diff = tdiff;
		type = P_IMIN;
	}
	if (diff > (tdiff = blk - cgtod(&disk->d_fs, i))) {
		diff = tdiff;
		type = P_TOD;
	}
	if (diff > (tdiff = blk - cgsblock(&disk->d_fs, i))) {
		diff = tdiff;
		type = P_SBLOCK;
	}

	switch (type) {
		case P_DMIN:
			printf("%llu into data\n", (unsigned long long)blk);
			/*
			 * scan inodes to find block 
			 */
			printf("last: %d\n",
					 disk->d_fs.fs_ncg * disk->d_fs.fs_ipg);
			for (i = 2; i < disk->d_fs.fs_ncg * disk->d_fs.fs_ipg; i++) {
				iblk = ino_to_fsba(&disk->d_fs, i);
				if (bread(disk, blk, inode, ilen) < (ssize_t) sizeof(struct ufs2_dinode)) {
					fprintf(stderr, "short read on inode %d, skipping\n", i);
					continue;
				}
				if (isblkininode(disk, blk, inode)) {
					*num = i;
					printf("found in inum: %d\n", i);
					break;
				}
			}
			printf("end: %d\n", i);
			break;
		case P_IMIN:
			printf("%llu into inode\n", (unsigned long long)blk);
			break;
		case P_SBLOCK:
			printf("%llu into super\n", (unsigned long long)blk);
			break;
		case P_TOD:
			printf("%llu into cylinder\n", (unsigned long long)blk);
			break;
	}
	free(inode);
}
