/*
 * Copyright 2005 Frank Mayhar.  All Rights Reserved.
 * Borrowed heavily from "ffsrecov," copyright 1999, 2001 John-Mark Gurney.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: ffs2recov.h,v 1.3 2007/02/06 05:21:50 frank Exp $
 *
 */

#include <ufs/ffs/fs.h>
#include <ufs/ufs/quota.h>
#include <ufs/ufs/inode.h>
#include <ufs/ufs/dir.h>

#define DI_SPARE
#define DI_INUMBER
#define DI_NSEC
#define FS_INODEFMT
#define REAL_QUAD_T

#define PF_LLONG	"q"

/* lets use a default block size of 64k for basic operations */
/* must be a multiple of DEV_BSIZE */
/* right now ata can transfer 128k at once, make use of this */
#define INT_BLKSIZE	(256 * DEV_BSIZE)

#define DPRINTF(x) if (debug) fprintf x
/* #define DPRINTF(x) */

/* I/O functions. */
#ifndef NO_IO_EXTERNS
extern size_t readin(struct uufsd *, void *data, off_t, size_t);
extern void *areadin(struct uufsd *, ufs2_daddr_t, size_t);
extern int writeout(struct uufsd *, int, off_t, size_t);
extern int bpread(int fd, void *, size_t, off_t);
#endif /* NO_IO_EXTERNS */

/* Superblock functions. */
#ifndef NO_SB_EXTERNS
extern void checksbcg(struct uufsd *, off_t);
extern void findsuperblks(struct uufsd *);
extern void printsuper(struct fs *fs);
extern int dumpfs(struct uufsd *);
extern int get_sblock(struct uufsd *, off_t);
#endif

/* Block functions. */
#ifndef NO_BL_EXTERNS
extern off_t blocktooffset(struct uufsd *, ufs2_daddr_t);
extern void findblk(struct uufsd *disk, ufs2_daddr_t blk, ino_t *num);
#endif

/* Inode functions. */
#ifndef NO_IN_EXTERNS
extern off_t inotooff(struct uufsd *, ino_t);
extern uint64_t inotodb(struct uufsd *, ino_t);
extern int getinode(struct uufsd *, void *, struct ufs2_dinode *);
extern int getinodeblock(struct uufsd *, void *, struct ufs2_dinode *, uint64_t);
extern void duplicateinode(struct uufsd *, ino_t, const char *);
extern int inodetofile(struct uufsd *, struct ufs2_dinode *, const char *);
extern int inodetofd(struct uufsd *, struct ufs2_dinode *, int);
extern int isblkininode(struct uufsd *, ufs2_daddr_t, struct ufs2_dinode *);
#endif

extern int mypow(int a, int b);

/* more generic functions */
extern void printdinode(struct ufs2_dinode *);

extern void printdirect(struct direct *dir, int);
extern ino_t *matchname(struct uufsd *, struct ufs2_dinode *, char *);
extern void searchfsforname(struct uufsd *, char *);
extern void printdirs(struct uufsd *);
extern void dumpoutinode(struct uufsd *, ino_t);
extern void dumpall(struct uufsd *);
extern void printinodeinfo(struct uufsd *, ino_t);
extern const char *typestr(int);
